#####################
#####Multiple Comparison for Probit model Based on Univariate Composte Likelihood
######################
######################

library(MASS)
library(mvtnorm)

set.seed(123466)
m    <- 4   
p    <- 10

rhox <- 0

n    <- 500
B    <- 10000
#############################Functions

fun1_b <- function(s,x,y){  
  ps  <- as.numeric(pnorm(s,0,1,lower.tail=TRUE))
  pms  <- as.numeric(pnorm(-s,0,1,lower.tail=TRUE))
  ds  <- as.numeric(dnorm(s,0,1))  
  if  (abs(s)<= 5) {z1 <- ((y-ps)*(ds*x))/(ps*pms)
  } else if (s>5 & y==1) {z1 <- ds*x/ps
  } else if (s>5 & y==0) {z1 <- -s*x
  } else if (s< -5 & y==1) {z1 <- -s*x
  } else if (s< -5 & y==0) {z1 <- -ds*x/pms}
  return(z1)
}


fun2_b <- function(s,x,y){  
  ps  <- as.numeric(pnorm(s,0,1,lower.tail=TRUE))
  pms  <- as.numeric(pnorm(-s,0,1,lower.tail=TRUE))
  ds  <- as.numeric(dnorm(s,0,1))
  if  (y==1 & s>-5 ) {z2 <-as.numeric((-ds^2/ps^2) - (ds*s/ps))*x%*%t(x)
  }else if (y==1 & s< -5) {z2 <- 0
  }else if (y==0 & s< 5 ) {z2 <-as.numeric((-ds^2/pms^2) + (ds*s/pms))*x%*%t(x)
  }else if (y==0 & s> 5) {z2 <- 0}
  return(z2)
}

#####################################################
sigma <- 1
sigx      <- array(rep(rhox,m^2),dim=c(m,m))
sigma_true<- sigma*(sigx+(1-rhox)*diag(m))

sM <-rep(0,B)
sB <-rep(0,B)
sT <-rep(0,B)
sD <-rep(0,B)
sH <-rep(0,B)
sS <-rep(0,B)
sN <-rep(0,B)

Mm <-rep(0,B)
Bb <-rep(0,B)
Tt <-rep(0,B)
Dd <-rep(0,B)
Hh <-rep(0,B)
Ss <-rep(0,B)
Nn <-rep(0,B)

if (p==10){
  C <- rbind(c(-1, 1, rep(0,8)),
             c(-1, 0, 1,rep(0,7)),
             c(-1, 0, 0, 1,rep(0,6)),
             c(-1, 0, 0, 0, 1,rep(0,5)),
             c(-1, 0, 0, 0, 0, 1,rep(0,4)),
             c(-1, 0, 0, 0, 0, 0, 1,rep(0,3)),
             c(-1, 0, 0, 0, 0, 0, 0, 1,rep(0,2)),
             c(-1, rep(0,7), 1, 0),
             c(-1, rep(0,8), 1))
  
}else if (p==20){
  
  
  C <- rbind(c(-1, 1, rep(0,18)),
             c(-1, 0, 1,rep(0,17)),
             c(-1, 0, 0, 1,rep(0,16)),
             c(-1, 0, 0, 0, 1,rep(0,15)),
             c(-1, 0, 0, 0, 0, 1,rep(0,14)),
             c(-1, 0, 0, 0, 0, 0, 1,rep(0,13)),
             c(-1, 0, 0, 0, 0, 0, 0, 1,rep(0,12)),
             c(-1, rep(0,7), 1, rep(0,11)),
             c(-1, rep(0,8), 1, rep(0,10)),
             c(-1, rep(0,9), 1, rep(0,9)),
             c(-1, rep(0,10), 1, rep(0,8)),
             c(-1, rep(0,11), 1, rep(0,7)),
             c(-1, rep(0,12), 1, rep(0,6)),
             c(-1, rep(0,13), 1, rep(0,5)),
             c(-1, rep(0,14), 1, rep(0,4)),
             c(-1, rep(0,15), 1, rep(0,3)),
             c(-1, rep(0,16), 1, rep(0,2)),
             c(-1, rep(0,17), 1, 0),
             c(-1, rep(0,18), 1))
}


#true_beta <- rep(0,p)
#true_beta <- c(0,0,0,0.03,rep(0,(p-4)))
true_beta <- c(0,0.008,0.01,-0.03,0.005, -0.01,rep(0,(p-6)))

seed <- sample(seq(111311,9999999,27),B)


for(b in 1:B){
  set.seed(seed[b])
  y_str         <- matrix(0,nrow=m,ncol=n)
  mu        <- array(dim=c(m,n))
  desx      <- array(dim=c(m,p,n))
  
  # Generating n families of size m from m-variate normal
  
  for (k in 1:n){
    for(j in 1:p){
      l <- sample(seq(-3,3,0.5),1)
      v <- sample(seq(3,6,0.6),1)
      desx[,j,k] <- rnorm(m,l,v)
     
    }
    mu[,k]       <- desx[,,k] %*% true_beta
    y_str[,k]    <- mvrnorm(1,mu[,k], sigma_true, tol = 1e-6, empirical = FALSE)
  }
  
  y  <- matrix(0,nrow=m,ncol=n)
  y <- ifelse(y_str>0 ,1 ,0)
  
  ###################################Initial value
  
  yy<- rep(0,n*m)
  for( i in 1:(n)){
    yy[((i-1)*m+1):(i*m)] <- y[,i]
  }
  xx<- matrix(0,nrow=m*n, ncol=p)
  data <- data.frame(xx,yy)
  for( i in 1:(n)){
    xx[((i-1)*m+1):(i*m),] <- desx[,,i]
  }
  
  fit <-glm(yy~ xx ,family=binomial(link = "probit"), data , 
            control = list(epsilon = 1e-6, maxit = 30, trace = FALSE))
  
  
  beta_hat <- as.vector(fit$coefficients[-1])
  
  #####################
  #######################Newton Raphson for beta
  
  sig <- 1
  
  for (k in 1:1000){ 
    beta_hat_n <- beta_hat 
    sc_b <-rep(0,p)
    hs_b <-matrix(0,p,p)
    for (i in 1:n){
      for( j in 1:m){
        
        X <-  desx[j,,i]%*%beta_hat/sig
        
        sc_b <- sc_b + fun1_b(X,desx[j,,i],y[j,i])
        hs_b <- hs_b + fun2_b(X,desx[j,,i],y[j,i])
      }
    }
    beta_hat <- ifelse(sig*abs(solve(hs_b)%*%sc_b) < 10^(-5),beta_hat_n,beta_hat_n - sig*solve(hs_b)%*%sc_b)
    #print(k)
    if (all(beta_hat== beta_hat_n)){break}
  } 
  
  
  #############################Var(S_1)
  
  v_s <- matrix(0,p,p)
  H <- matrix(0,p,p)
  
  
  for ( i in 1:n){
    dmu <- as.vector(dnorm(desx[,,i]%*%beta_hat))*desx[,,i]
    mu <- pnorm(desx[,,i]%*%beta_hat)
    V <- diag(as.vector(pnorm(desx[,,i]%*%beta_hat))*as.vector(pnorm(-desx[,,i]%*%beta_hat)))
    sij <-(y[,i]-mu)%*%t(y[,i]-mu)
    
    
    v_s <- v_s + t(dmu)%*%solve(V)%*%sij%*%solve(V)%*%dmu
    H <- H + t(dmu)%*%solve(V)%*%dmu
    
  }
  
  Cov_beta <- solve(H)%*%v_s%*%solve(H)
  
  
   
  bstr <- C%*%beta_hat
  cov_str <- C%*%Cov_beta%*%t(C)
  se_str <- sqrt(diag(cov_str))^(-1)
  v_mat  <- diag(se_str)
  corr_beta <- v_mat %*% cov_str %*% v_mat
  
  ################Cov(T), 
  T <- bstr*se_str
  cov_T <- v_mat%*%cov_str%*%t(v_mat)

  
  qu <- qmvnorm(0.95, tail ="both.tails", mean = 0,  corr=cov_T)
  quantile <- qu$quantile
  
  sM[b]   <-   sum(abs(T)<quantile)
  Mm[b] <- as.numeric(abs(T)[3,]<quantile)
  #Mm[b] <- as.numeric(abs(T)[1,]<quantile)+ as.numeric(abs(T)[2,]<quantile)+ as.numeric(abs(T)[3,]<quantile)+ as.numeric(abs(T)[4,]<quantile)+ as.numeric(abs(T)[5,]<quantile)
  
  ############################################Bonferroni
  Bon <- 0.05/nrow(C)
  qBon <- qnorm(1-Bon/2, lower.tail = TRUE)  
  
  sB[b]    <- sum(abs(T)<qBon)
  Bb[b] <- as.numeric(abs(T)[3,]<qBon)
  #Bb[b] <- as.numeric(abs(T)[1,]<qBon)+ as.numeric(abs(T)[2,]<qBon)+ as.numeric(abs(T)[3,]<qBon)+ as.numeric(abs(T)[4,]<qBon)+ as.numeric(abs(T)[5,]<qBon)
  
  #################################Naive method: considering independence
  
  Cov_bata_naive <- solve(H)
  cov_str_naive  <- C%*%Cov_bata_naive %*%t(C)
  se_str_naive <- diag(cov_str_naive)^(-1/2)
  v_mat_n  <- diag(se_str_naive)
  corr_beta_naive <- v_mat_n %*% cov_str_naive %*% v_mat_n
  
  T_n <- bstr*se_str_naive
  cov_T_naive <- v_mat_n%*%cov_str_naive%*%t(v_mat_n)  
  
  qu_n <- qmvnorm(0.95, tail ="both.tails", mean = 0,  corr=cov_T_naive )####Simultaneous quantile
  quantile_naive <- qu_n$quantile
  
  sN[b]    <- 	sum(abs(T_n)<quantile_naive)
  Nn[b] <- as.numeric(abs(T_n)[3,]<quantile_naive)
  #Nn[b] <- as.numeric(abs(T_n)[1,]<quantile_naive)+ as.numeric(abs(T_n)[2,]<quantile_naive)+ as.numeric(abs(T_n)[3,]<quantile_naive)+ as.numeric(abs(T_n)[4,]<quantile_naive)+ as.numeric(abs(T_n)[5,]<quantile_naive)
  
  ###############################################Dunn-Sidak
  dunn <- 1-(1-0.05)^(1/nrow(C))
  qDunn <- qnorm(1-dunn/2)  
  sD[b] <- sum(abs(T)<qDunn)
  Dd[b] <- as.numeric(abs(T)[3,]<qDunn) 
  Dd[b] <-  as.numeric(abs(T)[1,]<qDunn)+ as.numeric(abs(T)[2,]<qDunn)+ as.numeric(abs(T)[3,]<qDunn)+ as.numeric(abs(T)[4,]<qDunn)+ as.numeric(abs(T)[5,]<qDunn)
  
  ##############################################Holm
  t_h <- sort(abs(T),decreasing=TRUE)
  Holm <- 0.05/(nrow(C)-c(1:nrow(C))+1)
  qHolm <- qnorm(1-Holm/2) 
  ind1 <- t_h< qHolm
  
  sH[b] <- as.numeric(ind1[1]==FALSE)##Global
  #   for ( i in 1:5){
  #     if (ind1[i]==FALSE){
  #       Hh[b] <- Hh[b] +1
  #       if (ind1[(i+1)]==TRUE){
  #         break}
  #     }
  #   }
  ### Scheffe
  
  qscheffe  <- sqrt(qchisq(0.95,(p-1)))  
  sS[b] <- sum(abs(T)< qscheffe )
  Ss[b] <- as.numeric(abs(T)[3,]<qscheffe)
  #Ss[b] <- as.numeric(abs(T)[1,]<qscheffe)+ as.numeric(abs(T)[2,]<qscheffe)+ as.numeric(abs(T)[3,]<qscheffe)+ as.numeric(abs(T)[4,]<qscheffe)+ as.numeric(abs(T)[5,]<qscheffe)
  
  ### Tukey
  
  qtuk     <- qtukey(0.05,p,(m*n-p),lower.tail = F)/sqrt(2) 
  sT[b]       <- sum( abs(T)< qtuk)
  Tt[b] <- as.numeric(abs(T)[3,]<qtuk)
  #Tt[b] <- as.numeric(abs(T)[1,]<qtuk)+ as.numeric(abs(T)[2,]<qtuk)+ as.numeric(abs(T)[3,]<qtuk)+ as.numeric(abs(T)[4,]<qtuk)+ as.numeric(abs(T)[5,]<qtuk)
  
  ##############################################
  
  answ1<-c(paste("m",m,"p",p,"r",rhox),"MNQ",sM[b],"Naive", sN[b], "Bon" ,sB[b],"Sidak",sD[b],"Holm",sH[b], "Scheffe",sS[b],"Tukey", sT[b])
  write.table(answ1 ,file= paste("F://PowerProbit/PrpowA", m,p,rhox,".txt"),append=TRUE,row.names=FALSE,col.names=FALSE,sep=" ",eol = "\n")
  
  
  answ2<- c(paste("m",m,"p",p,"r",rhox),"MNQ",Mm[b],"Naive", Nn[b], "Bon" ,Bb[b],"Sidak",Dd[b],"Holm",Hh[b], "Scheffe",Ss[b],"Tukey", Tt[b])
  write.table(answ2 ,file=paste("F://PowerProbit/Prpow2A", m,p,rhox,".txt"),append=TRUE,row.names=FALSE,col.names=FALSE,sep=" ",eol = "\n")
  
  
  print(b)
} 
