
library(MASS)
library(mvtnorm)

set.seed(223344)
p 	<- 	10 
m 	<- 	4
w 	<- 	0.5  

B     <- 50
n   <- 	700

true_beta  <-  rep(0,p)
#true_beta <- c(0,0,0,0.12,rep(0,(p-4)))
#true_beta <- c(0,0.08,0.12,-0.03,0.05, -0.08,rep(0,(p-6)))


sM <-rep(0,B)
sB <-rep(0,B)
sT <-rep(0,B)
sD <-rep(0,B)
sH <-rep(0,B)
sS <-rep(0,B)
sN <-rep(0,B)

Mm <-rep(0,B)
Bb <-rep(0,B)
Tt <-rep(0,B)
Dd <-rep(0,B)
Hh <-rep(0,B)
Ss <-rep(0,B)
Nn <-rep(0,B)

if (p==10){
 C <- rbind(c(-1, 1, rep(0,8)),
            c(-1, 0, 1,rep(0,7)),
            c(-1, 0, 0, 1,rep(0,6)),
            c(-1, 0, 0, 0, 1,rep(0,5)),
            c(-1, 0, 0, 0, 0, 1,rep(0,4)),
            c(-1, 0, 0, 0, 0, 0, 1,rep(0,3)),
            c(-1, 0, 0, 0, 0, 0, 0, 1,rep(0,2)),
            c(-1, rep(0,7), 1, 0),
            c(-1, rep(0,8), 1))

  }else if (p==20){


C <- rbind(c(-1, 1, rep(0,18)),
           c(-1, 0, 1,rep(0,17)),
           c(-1, 0, 0, 1,rep(0,16)),
           c(-1, 0, 0, 0, 1,rep(0,15)),
           c(-1, 0, 0, 0, 0, 1,rep(0,14)),
           c(-1, 0, 0, 0, 0, 0, 1,rep(0,13)),
           c(-1, 0, 0, 0, 0, 0, 0, 1,rep(0,12)),
           c(-1, rep(0,7), 1, rep(0,11)),
           c(-1, rep(0,8), 1, rep(0,10)),
           c(-1, rep(0,9), 1, rep(0,9)),
           c(-1, rep(0,10), 1, rep(0,8)),
           c(-1, rep(0,11), 1, rep(0,7)),
           c(-1, rep(0,12), 1, rep(0,6)),
           c(-1, rep(0,13), 1, rep(0,5)),
           c(-1, rep(0,14), 1, rep(0,4)),
           c(-1, rep(0,15), 1, rep(0,3)),
           c(-1, rep(0,16), 1, rep(0,2)),
           c(-1, rep(0,17), 1, 0),
           c(-1, rep(0,18), 1))
}

C   <-  t(C)


C 		<- 	matrix(0,nrow=p,ncol=p*(p-1)/2)
comb 	<- 	combn(p,2)
C[cbind(comb[1,],1:(p*(p-1)/2))] <- 1
C[cbind(comb[2,],1:(p*(p-1)/2))] <- -1

seed 	<- 	sample(seq(1111111,99999999,27),B)


###	Monte Carlo simulation to approximate familywise type I error and power

for (b in 1:B){
  
  set.seed(seed[b])
  x	  <-	array(rnorm(n*p,0,1),dim=c(n,p)) 

  
  cluster	<-	array(dim=c(n,2))					##matrix of cluster size and number of successes in each cluster
  cluster[,2]	<-	sample((4:8),n, replace=TRUE)
  
  
  for (i in 1:n){
    theta	<-	x[i,]%*%true_beta
    ni	<-	cluster[i,2]
    probz	<-	numeric(ni+1)
    z		<-	0:cluster[i,2]
    probz	<-	exp(theta*z-w*z*(ni-z))*choose(ni,z)
    probz	<-	probz/sum(probz)
    cluster[i,1]<-	which.max(rmultinom(1,1,probz))-1
  }
  
  
  ###	   adding a correlation structure as a new covariate 
  
  
  sbigpat	<-	array(0,dim=c(2*n,2))
  corw	<-	numeric(2*n)
  
  for (ct in 1:n){
    
    sbigpat[(ct*2)-1,]	<-	c(cluster[ct,1],0)
    sbigpat[(ct*2),]	<-	c(0,(cluster[ct,2]-cluster[ct,1]))
    corw[(ct*2)-1]	<-	-(cluster[ct,2]-2*cluster[ct,1]+1)
    corw[(ct*2)]		<-	-(cluster[ct,2]-2*cluster[ct,1]-1)
  }
  
  kidrep	<-	array(rep(c(x),each=2),dim=c(2*n,p))
  kidrep	<-	cbind(kidrep,corw)
  
  
  ###    building a binary response variable according to the number of success and failure
  
  
  binarysta	<-	NULL
  for (ct in 1:n){
    ss		<-	sbigpat[(2*ct-1),1]
    ff		<-	sbigpat[(2*ct),2]
    binarysta	<-	c(binarysta,rep(1,ss),rep(0,ff))
  }
  
  v1		<-	rep(c(1,0),time=n)
  y1		<-	apply(sbigpat,1,sum)
  binarysta <-	rep(v1,y1)
  convkid	<-	array(rep(c(kidrep),rep(y1,(p+1))),dim=c(sum(y1),(p+1)))
  biconvkid <-	cbind(binarysta,convkid)
  colnames(biconvkid)[2:(p+1)] <- 	paste("V",1:p,sep="")
  colnames(biconvkid)[p+2] 	 <-	"corw"
  biconvkid				 <-	data.frame(biconvkid)
  
  
  ###    fitting a ligistic regression to estimate the coefficient
  
  xnam 	<- 	paste("V", 1:p, sep="")
  fmla 	<- 	as.formula(paste("binarysta~corw+", paste(xnam, collapse= "+"),sep=""))
  res		<-	glm(fmla,data=biconvkid,family=binomial(link=logit))
  summary(res)
  beta 	<- 	res$coefficients[3:(p+2)]
  what 	<- 	res$coefficients[2]
  
  
  head(xdat,20)
  head(biconvkid,20)
  ###    Estimating score function
  
  
  xdes	<-	biconvkid[,-1]
  xdes 	<-	as.matrix(xdes)
  prob 	<-	numeric(2*n)
  
  for ( i in 1: n){
    es 	<- 	exp(x[i,]%*%beta-what*(cluster[i,2]-2*cluster[i,1]+1))
    ef 	<- 	exp(-x[i,]%*%beta+what*(cluster[i,2]-2*cluster[i,1]-1))
    prob[(2*i-1)] <-  es/(1+es)
    prob[(2*i)] 	<-  ef/(1+ef) 
  }
  
  scorevec	<-	array(0,dim=c(n,(p+1)))
  
  ind1	<-	seq(1,(2*n),2)
  ind2	<-	ind1+1
  
  subma1	<-	kidrep[ind1,]
  subma2	<-	kidrep[ind2,]
  bprob1	<-	sbigpat[ind1,1]*(1-prob[ind1])
  bprob2	<-	sbigpat[ind2,2]*(1-prob[ind2])
  scorevec	<-	subma1*bprob1-subma2*bprob2
  
  
  
  ###    estimating Godambe information matrix
  
  
  j		<-	n*var(scorevec)[-(p+1),-(p+1)]
  h		<-	(t(xdes)%*%diag(res$weights)%*%xdes)[-(p+1),-(p+1)] 
  Cov_beta	<-	(solve(h)%*%j%*%solve(h))#[-(p+1),-(p+1)]
  
  
  
  
  ###    Computing the test statistic
  
  bstr 	<- 	t(C)%*%beta
  cov_str 	<- 	t(C)%*%Cov_beta%*%C
  se_str 	<- 	diag(cov_str)^(-1/2)
  v_mat  	<- 	diag(se_str)  
  corr_beta <- 	v_mat %*% cov_str %*% v_mat
  
  
  T 		<- 	bstr*se_str
  cov_T 	<- 	v_mat%*%cov_str%*%t(v_mat)
  
  
  ###	computing the m-variate normal quantile to consider it as threshold and performing hypothesis test	
  
  qu 		<- 	qmvnorm(0.95, tail ="both.tails", mean = 0,  sigma=cov_T)
  quantile  <- 	qu$quantile
  sM[b]   <-   sum(abs(T)<quantile)
  Mm[b] <- as.numeric(abs(T)[3,]<quantile)
  #Mm[b] <- as.numeric(abs(T)[1,]<quantile)+ as.numeric(abs(T)[2,]<quantile)+ as.numeric(abs(T)[3,]<quantile)+ as.numeric(abs(T)[4,]<quantile)+ as.numeric(abs(T)[5,]<quantile)
  
  ###     Naive method: ignoring intra-cluster correlation
  
  Cov_bata_naive 	<- 	solve(h)#[-(p+1),-(p+1)])
  cov_str_naive  	<- 	t(C)%*%Cov_bata_naive %*%C
  se_str_naive 	<- 	diag(cov_str_naive)^(-1/2)
  v_mat_n  		<- 	diag(se_str_naive)
  corr_beta_naive <- 	v_mat_n %*% cov_str_naive %*% v_mat_n
  T_n 		<- 	bstr*se_str_naive
  cov_T_naive 	<- 	v_mat_n%*%cov_str_naive%*%t(v_mat_n)    
  qu_n 		<- 	qmvnorm(0.95, tail ="both.tails", mean = 0,  sigma=cov_T_naive )
  quantile_naive 	<- 	qu_n$quantile
  
  sN[b]    <- 	sum(abs(T_n)<quantile_naive)
  Nn[b] <- as.numeric(abs(T_n)[3,]<quantile_naive)
  #Nn[b] <- as.numeric(abs(T_n)[1,]<quantile_naive)+ as.numeric(abs(T_n)[2,]<quantile_naive)+ as.numeric(abs(T_n)[3,]<quantile_naive)+ as.numeric(abs(T_n)[4,]<quantile_naive)+ as.numeric(abs(T_n)[5,]<quantile_naive)
  
  ###  Bonferroni  
  
  Bon 	<- 	0.05/ncol(C)
  qBon 	<- 	qnorm(1-Bon/2)  
  sB[b]    <- sum(abs(T)<qBon)
  Bb[b] <- as.numeric(abs(T)[3,]<qBon)
  #Bb[b] <- as.numeric(abs(T)[1,]<qBon)+ as.numeric(abs(T)[2,]<qBon)+ as.numeric(abs(T)[3,]<qBon)+ as.numeric(abs(T)[4,]<qBon)+ as.numeric(abs(T)[5,]<qBon)
  
  ###	Dunn-Sidak
  
  dunn 		 <- 	1-(1-0.05)^(1/ncol(C))
  qDunn 		 <- 	qnorm(1-dunn/2)  
  sD[b] <- sum(abs(T)<qDunn)
  Dd[b] <- as.numeric(abs(T)[3,]<qDunn) 
  #Dd[b] <-  as.numeric(abs(T)[1,]<qDunn)+ as.numeric(abs(T)[2,]<qDunn)+ as.numeric(abs(T)[3,]<qDunn)+ as.numeric(abs(T)[4,]<qDunn)+ as.numeric(abs(T)[5,]<qDunn)
  
  ###	Holm
  
  t_h   	 <- 	sort(abs(T),decreasing=TRUE)
  Holm 		 <- 	0.05/(ncol(C)-c(1:ncol(C))+1)
  qHolm 		 <- 	qnorm(1-Holm/2) 
  
  ind1 <- t_h< qHolm
  
  sH[b] <- as.numeric(ind1[1]==FALSE)##Global
  #   for ( i in 1:5){
  #     if (ind1[i]==FALSE){
  #       Hh[b] <- Hh[b] +1
  #       if (ind1[(i+1)]==TRUE){
  #         break}
  #     }
  #   }

  
  ### Scheffe
  qscheffe  <- sqrt(qchisq(0.95,(p-1)))  
  sS[b] <- sum(abs(T)< qscheffe )
  Ss[b] <- as.numeric(abs(T)[3,]<qscheffe)
  #Ss[b] <- as.numeric(abs(T)[1,]<qscheffe)+ as.numeric(abs(T)[2,]<qscheffe)+ as.numeric(abs(T)[3,]<qscheffe)+ as.numeric(abs(T)[4,]<qscheffe)+ as.numeric(abs(T)[5,]<qscheffe)
  
  ### Tukey
  qtuk     <- qtukey(0.05,p,(m*n-p),lower.tail = F)/sqrt(2)  
  sT[b]       <- sum( abs(T)< qtuk)
  Tt[b] <- as.numeric(abs(T)[3,]<qtuk)
  #Tt[b] <- as.numeric(abs(T)[1,]<qtuk)+ as.numeric(abs(T)[2,]<qtuk)+ as.numeric(abs(T)[3,]<qtuk)+ as.numeric(abs(T)[4,]<qtuk)+ as.numeric(abs(T)[5,]<qtuk)
  
  
  answ1<-c(paste("m",m,"p",p,"r",w),"MNQ",sM[b],"Naive", sN[b], "Bon" ,sB[b],"Sidak",sD[b],"Holm",sH[b], "Scheffe",sS[b],"Tukey", sT[b])
  write.table(answ1 ,file= paste("d://Xin/mahdis/COR_files/PowerQEAll/QEpowA2", m,p,w,".txt"),append=TRUE,row.names=FALSE,col.names=FALSE,sep=" ",eol = "\n")
  
  
  answ2<- c(paste("m",m,"p",p,"r",w),"MNQ",Mm[b],"Naive", Nn[b], "Bon" ,Bb[b],"Sidak",Dd[b],"Holm",Hh[b], "Scheffe",Ss[b],"Tukey", Tt[b])
  write.table(answ2 ,file=paste("d://Xin/mahdis/COR_files/PowerQEAll/QEpow2A2", m,p,w,".txt"),append=TRUE,row.names=FALSE,col.names=FALSE,sep=" ",eol = "\n")
  
  print(b)
  
} 

