library(mvtnorm)

dt1  <-  read.csv("cl_hrs.csv", header = TRUE)
dt   <-  as.data.frame(dt1[,-2])


####first writing in long format then removing the NA's

d1   <-  dt[,c(1:11)]#depression
d2   <-  dt[,c(1,22:31)]#sleep
d3   <-  dt[,c(1,33:42)]#diabete
d4   <-  dt[,c(1,66:75)]#smoke
d5   <-  dt[,c(1,77:86)]#age
d6   <-  dt[,c(1,88:97)]#activity
d7   <-  dt[,c(1,131:140)]#hibp
d8   <-  dt[,c(1,141:150)]#walk


d1l <-  reshape(d1, varying=2:11, v.names = "depress",timevar = "..hhidpn",
               direction = "long")
d2l <-  reshape(d2, varying=2:11, v.names = "sleep",timevar = "..hhidpn",
               direction = "long")
d3l <-  reshape(d3, varying=2:11, v.names = "diabetes",timevar = "..hhidpn",
               direction = "long")
d4l <-  reshape(d4, varying=2:11, v.names = "smoke",timevar = "..hhidpn",
               direction = "long")
d5l <-  reshape(d5, varying=2:11, v.names = "age",timevar = "..hhidpn",
               direction = "long")
d6l <-  reshape(d6, varying=2:11, v.names = "activity",timevar = "..hhidpn",
               direction = "long")
d7l <-  reshape(d7, varying=2:11, v.names = "hibp",timevar = "..hhidpn",
                direction = "long")
d8l <-  reshape(d8, varying=2:11, v.names = "walk",timevar = "..hhidpn",
                direction = "long")

mdt <- Reduce(function(x, y) merge(x, y, all=TRUE), list(d1l,d2l,d3l, d4l,d5l,d6l, d7l,d8l))


###Removing the rows with missing value

ss2   <-  apply(mdt,1,function(x)sum(!is.na(x)))
dt2   <-  cbind(mdt,ss2)
mdt3  <-  dt2[ss2 ==ncol(mdt),]
dt3   <-  as.data.frame(mdt3[,-c(1,ncol(mdt3))])
sdt   <-  dt3[order(dt3$id),  ]    #The final data set used in the analysis


N   <-  length((sdt$id))              #numner of measurements
n   <-  length(unique(sdt$id))       #number of individuals
p   <-  ncol(sdt)-2                   # number of factors

N
n
p

## creating the column of dependence factor (w)

pat      <-  array(0,dim=c(n,2))
pat[,2]  <-  table(sdt$id)
ct       <-  1
breakdown<-  vector("list", n)

for (i in 1:(N-1)){
  breakdown[[ct]]  <- c(breakdown[[ct]],sdt$depres[i])
  if (sdt$id[i+1]>sdt$id[i]){ct<-ct+1}
}
for (i in N:N){ 
  breakdown[[ct]]  <- c(breakdown[[ct]],sdt$depres[i])
}
for ( i in 1: n){
  pat[i,1]         <- sum(breakdown[[i]])
}

apply(pat,2,sum)
ss    <-  pat[,1]
ff    <-  pat[,2]
sdt$z <-  rep(ss,ff)
sdt$m <-  rep(ff,ff)
sdt$w <-  ifelse(sdt$depres==1,  -(sdt$m-2*sdt$z+1),-(sdt$m-2*sdt$z-1))
sdt$age <- sdt$age/12

r     <-  dim(sdt)[2]
xdat  <-  sdt[,-c(1,(r-1),(r-2))] 

factnam  <-  names(xdat[,-1])
resp     <-  xdat$depres

fmla     <-  as.formula(paste("resp ~ ", paste(factnam, collapse= "+")))
fit      <-  glm(fmla ,data=xdat ,family=binomial(link=logit))
beta_hat <-  fit$coefficients[-1]
summary(fit)
beta    <- 	as.matrix(fit$coefficients[2:(p+1)])
what 	  <- 	fit$coefficients[(p+2)]


X <- split(sdt, sdt$id)
X <- lapply(X, function(x) { x[c("id")] <- NULL; x })
X <- lapply(X, function(x) { x[c("w")] <- NULL; x })


scorevec  <-    array(0,dim=c(n,p))

for ( i in 1: n){
  
  xxs      <-    as.data.frame(X[[i]])
  xxs$indc <-    ifelse(xxs$depress==1, 1, -1)
  e        <-    rep(0,nrow(xxs))
  
  for(q in 1: nrow(xxs)){
    
    e[q] <- ifelse(xxs$depress[q]==1,exp(as.matrix(xxs[q,-c(1,9,10,11)])%*%beta-what*(xxs$m[q]-2*xxs$z[q]+1))
                       ,exp(-as.matrix(xxs[q,-c(1,9,10,11)])%*%beta +what*(xxs$m[q]-2*xxs$z[q]-1)) ) 
  }
  
  xxj           <- cbind(xxs,e)
  xxj$prob      <- 1/(1+e)
  multmat       <- xxj[,-c(1,9:13)]*xxj$prob*xxj$indc
  scorevec[i,]  <- apply(multmat,2,sum)
}


xdes     <-  xdat[,-c(1,ncol(xdat))]
xdes     <-  as.matrix(xdes)
dim(xdes)
head(xdes)


vmat     <-  var(scorevec)
hmat     <-  t(xdes*(fit$weights))%*%xdes
Cov_beta <-  n*solve(hmat)%*%vmat%*%solve(hmat)

std_bet  <- sqrt(diag(Cov_beta))

Z        <- beta/std_bet
pValue   <- 2*pnorm(-abs(Z))  

###############################all pair wise comparison
C                                 <-   matrix(0,nrow=p,ncol=p*(p-1)/2)
comb                              <-   combn(p,2)
C[cbind(comb[1,],1:(p*(p-1)/2))]  <-   1
C[cbind(comb[2,],1:(p*(p-1)/2))]  <-  -1
C                                 <-  t(C)

bstr      <-   C%*%beta
cov_str   <-   C%*%Cov_beta%*%t(C)
se_str    <-   diag(cov_str)^(-1/2)
v_mat     <-   diag(se_str)
corr_beta <-   v_mat %*% cov_str %*% v_mat
qu        <-   qmvnorm(0.95, tail ="both.tails", mean = 0, corr=corr_beta)# corr= corr_beta)####Simultaneous quantile
quantile  <-   qu$quantile
T         <-   bstr*se_str
res       <-   abs(T)<quantile
which(res==FALSE)


#################################Naive method: considering independence

Cov_bata_naive   <-   solve(hmat)
cov_str_naive    <-   C%*%Cov_bata_naive %*%t(C)
se_str_naive     <-   diag(cov_str_naive)^(-1/2)
v_mat_n          <-   diag(se_str_naive)
corr_beta_naive  <-  v_mat_n %*% cov_str_naive %*% v_mat_n
qu_n             <-  qmvnorm(0.95, tail ="both.tails", mean = 0,  corr= corr_beta_naive)####Simultaneous quantile
quantile_naive   <-  qu_n$quantile
T_n              <-  bstr*se_str_naive
res_naive        <-  abs(T_n)<quantile_naive
which(res_naive==FALSE)
